@@ -6,7 +6,7 @@
 #    =====))=))===() 
 #      ///'       
 #     //
-#    '            
+#    '
 from datetime import datetime
 import logging
 import os
@@ -229,6 +229,7 @@ class BaseQuery(object):
     }
     query_separator = '__'
     requires_commit = True
+    force_alias = False
     
     def __init__(self, model):
         self.model = model
@@ -244,11 +245,24 @@ class BaseQuery(object):
                 lhs, op = lhs.rsplit(self.query_separator, 1)
             else:
                 op = 'eq'
-
-            field = model._meta.get_field_by_name(lhs)
+            
+            try:
+                field = model._meta.get_field_by_name(lhs)
+            except AttributeError:
+                field = model._meta.get_related_field_by_name(lhs)
+                if field is None:
+                    raise
+                if isinstance(rhs, Model):
+                    rhs = rhs.id
+            
             if op == 'in':
-                lookup_value = [field.lookup_value(op, o) for o in rhs]
-                operation = self.operations[op] % (','.join(['?' for v in lookup_value]))
+                if isinstance(rhs, SelectQuery):
+                    lookup_value = rhs
+                    operation = 'IN (%s)'
+                else:
+                    lookup_value = [field.lookup_value(op, o) for o in rhs]
+                    operation = self.operations[op] % \
+                        (','.join(['?' for v in lookup_value]))
             else:
                 lookup_value = field.lookup_value(op, rhs)
                 operation = self.operations[op]
@@ -264,9 +278,9 @@ class BaseQuery(object):
         return self
     
     @mark_query_dirty
-    def join(self, model, join_type=None):
+    def join(self, model, join_type=None, on=None):
         if self.query_context._meta.rel_exists(model):
-            self._joins.append((model, join_type))
+            self._joins.append((model, join_type, on))
             self.query_context = model
         else:
             raise AttributeError('No foreign key found between %s and %s' % \
@@ -274,8 +288,8 @@ class BaseQuery(object):
         return self
     
     def use_aliases(self):
-        return len(self._joins) > 0
-    
+        return len(self._joins) > 0 or self.force_alias
+
     def combine_field(self, alias, field_name):
         if alias:
             return '%s.%s' % (alias, field_name)
@@ -289,13 +303,13 @@ class BaseQuery(object):
 
         joins = list(self._joins)
         if self._where or len(joins):
-            joins.insert(0, (self.model, None))
+            joins.insert(0, (self.model, None, None))
         
         where_with_alias = []
         where_data = []
         computed_joins = []
 
-        for i, (model, join_type) in enumerate(joins):
+        for i, (model, join_type, on) in enumerate(joins):
             if alias_required:
                 alias_count += 1
                 alias_map[model] = 't%d' % alias_count
@@ -304,12 +318,12 @@ class BaseQuery(object):
             
             if i > 0:
                 from_model = joins[i-1][0]
-                field = from_model._meta.get_related_field_for_model(model)
+                field = from_model._meta.get_related_field_for_model(model, on)
                 if field:
                     left_field = field.name
                     right_field = 'id'                        
                 else:
-                    field = from_model._meta.get_reverse_related_field_for_model(model)
+                    field = from_model._meta.get_reverse_related_field_for_model(model, on)
                     left_field = 'id'
                     right_field = field.name
                 
@@ -328,27 +342,26 @@ class BaseQuery(object):
                         self.combine_field(alias_map[model], right_field),
                     )
                 )
-
-        for (model, join_type) in joins:
-            if model in self._where:
-                for node in self._where[model]:
-                    query, data = self.parse_node(node, model, alias_map[model])
-                    where_with_alias.append(query)
-                    where_data.extend(data)
+        
+        for model in self._where:
+            for node in self._where[model]:
+                query, data = self.parse_node(node, model, alias_map)
+                where_with_alias.append(query)
+                where_data.extend(data)
         
         return computed_joins, where_with_alias, where_data, alias_map
     
-    def parse_node(self, node, model, alias):
+    def parse_node(self, node, model, alias_map):
         query = []
         query_data = []
         nodes = []
         for child in node.children:
             if isinstance(child, Q):
-                parsed, data = self.parse_q(child, model, alias)
+                parsed, data = self.parse_q(child, model, alias_map)
                 query.append(parsed)
                 query_data.extend(data)
             elif isinstance(child, Node):
-                parsed, data = self.parse_node(child, model, alias)
+                parsed, data = self.parse_node(child, model, alias_map)
                 query.append('(%s)' % parsed)
                 query_data.extend(data)
         query.extend(nodes)
@@ -358,14 +371,19 @@ class BaseQuery(object):
             query = 'NOT (%s)' % query
         return query, query_data
     
-    def parse_q(self, q, model, alias):
+    def parse_q(self, q, model, alias_map):
         query = []
         query_data = []
         parsed = self.parse_query_args(model, **q.query)
         for (name, lookup) in parsed.iteritems():
             operation, value = lookup
+            if isinstance(value, SelectQuery):
+                sql, value = self.convert_subquery(value)
+                operation = operation % sql
+
             query_data.append(value)
-            combined = self.combine_field(alias, name)
+            
+            combined = self.combine_field(alias_map[model], name)
             query.append('%s %s' % (combined, operation))
         
         if len(query) > 1:
@@ -377,27 +395,26 @@ class BaseQuery(object):
             query = 'NOT %s' % query
         
         return query, query_data
+
+    def convert_subquery(self, subquery):
+        subquery.query, orig_query = 'id', subquery.query
+        subquery.force_alias, orig_alias = True, subquery.force_alias
+        sql, data = subquery.sql()
+        subquery.query = orig_query
+        subquery.force_alias = orig_alias
+        return sql, data
     
     def raw_execute(self):
         query, params = self.sql()
-        db = self.model._meta.get_database()
+        db = self.model._meta.database
         result = db.execute(query, params, self.requires_commit)
         return result
 
 
 class SelectQuery(BaseQuery):
-    """
-    Model.select('*').where(field=val).join(RelModel).where(rel_field=val)
-    """
     requires_commit = False
     
     def __init__(self, model, query=None):
-        """
-        Allow a string or a dictionary keyed by model->fields
-        
-        .select('t1.*, COUNT(t2.id) AS count') or
-        .select({Blog: '*', Entry: Count('id')})
-        """
         self.query = query or '*'
         self._group_by = []
         self._having = []
@@ -423,7 +440,7 @@ class SelectQuery(BaseQuery):
         else:
             self.query = 'COUNT(id)'
         
-        db = self.model._meta.get_database()
+        db = self.model._meta.database
         res = db.execute(*self.sql())
         
         self.query = tmp_query
@@ -461,8 +478,8 @@ class SelectQuery(BaseQuery):
 
     def parse_select_query(self, alias_map):
         if isinstance(self.query, basestring):
-            if self.query == '*' and self.use_aliases():
-                return '%s.*' % alias_map[self.model]
+            if self.query in ('*', 'id') and self.use_aliases():
+                return '%s.%s' % (alias_map[self.model], self.query)
             return self.query
         elif isinstance(self.query, dict):
             qparts = []
@@ -555,9 +572,6 @@ class SelectQuery(BaseQuery):
 
 
 class UpdateQuery(BaseQuery):
-    """
-    Model.update(field=val, field2=val2).where(some_field=some_val)
-    """
     def __init__(self, model, **kwargs):
         self.update_query = kwargs
         super(UpdateQuery, self).__init__(model)
@@ -602,9 +616,6 @@ class UpdateQuery(BaseQuery):
 
 
 class DeleteQuery(BaseQuery):
-    """
-    Model.delete().where(some_field=some_val)
-    """
     def sql(self):
         joins, where, where_data, alias_map = self.compile_where()
 
@@ -630,9 +641,6 @@ class DeleteQuery(BaseQuery):
 
 
 class InsertQuery(BaseQuery):
-    """
-    Model.insert(field=val, field2=val2)
-    """
     def __init__(self, model, **kwargs):
         self.insert_query = kwargs
         super(InsertQuery, self).__init__(model)
@@ -741,6 +749,7 @@ class DateTimeField(Field):
     
     def python_value(self, value):
         if value is not None:
+            value = value.rsplit('.', 1)[0]
             return datetime(*time.strptime(value, '%Y-%m-%d %H:%M:%S')[:6])
     
     def db_value(self, value):
@@ -806,9 +815,10 @@ class ReverseForeignRelatedObject(object):
 class ForeignKeyField(IntegerField):
     field_template = '%(db_field)s %(nullable)s REFERENCES "%(to_table)s" ("id")'
     
-    def __init__(self, to, null=False, *args, **kwargs):
+    def __init__(self, to, null=False, related_name=None, *args, **kwargs):
         self.to = to
         self.null = null
+        self.related_name = related_name
         kwargs['to_table'] = to._meta.db_table
         if null:
             kwargs['nullable'] = ''
@@ -819,7 +829,10 @@ class ForeignKeyField(IntegerField):
     def add_to_class(self, klass, name):
         self.descriptor = name
         self.name = name + '_id'
-        self.related_name = klass._meta.db_table + '_set'
+        if self.related_name is None:
+            self.related_name = klass._meta.db_table + '_set'
+        
+        klass._meta.rel_fields[name] = self.name
         setattr(klass, self.descriptor, ForeignRelatedObject(self.to, self.name))
         setattr(klass, self.name, None)
         
@@ -836,29 +849,36 @@ class BaseModel(type):
     def __new__(cls, name, bases, attrs):
         cls = super(BaseModel, cls).__new__(cls, name, bases, attrs)
 
+        meta_attrs = attrs.pop('Meta', None)
+
         class Meta(object):
+            rel_fields = {}
             fields = {}
+            database = database
             
             def __init__(self, model_class):
                 self.model_class = model_class
-
-            def get_database(self):
-                return self.model_class.database
             
             def get_field_by_name(self, name):
                 if name in self.fields:
                     return self.fields[name]
                 raise AttributeError('Field named %s not found' % name)
             
-            def get_related_field_for_model(self, model):
+            def get_related_field_by_name(self, name):
+                if name in self.rel_fields:
+                    return self.fields[self.rel_fields[name]]
+            
+            def get_related_field_for_model(self, model, name=None):
                 for field in self.fields.values():
                     if isinstance(field, ForeignKeyField) and field.to == model:
-                        return field
+                        if name is None or name == field.name:
+                            return field
             
-            def get_reverse_related_field_for_model(self, model):
+            def get_reverse_related_field_for_model(self, model, name=None):
                 for field in model._meta.fields.values():
                     if isinstance(field, ForeignKeyField) and field.to == self.model_class:
-                        return field
+                        if name is None or name == field.name:
+                            return field
             
             def rel_exists(self, model):
                 return self.get_related_field_for_model(model) or \
@@ -866,6 +886,11 @@ class BaseModel(type):
             
         
         _meta = Meta(cls)
+
+        if meta_attrs:
+            for k, v in meta_attrs.__dict__.items():
+                setattr(_meta, k, v)
+
         setattr(cls, '_meta', _meta)
         
         _meta.db_table = re.sub('[^a-z]+', '_', cls.__name__.lower())
@@ -893,7 +918,6 @@ class BaseModel(type):
 
 class Model(object):
     __metaclass__ = BaseModel
-    database = database
     
     def __init__(self, *args, **kwargs):
         for k, v in kwargs.items():
@@ -909,11 +933,11 @@ class Model(object):
     
     @classmethod
     def create_table(cls):
-        cls.database.create_table(cls)
+        cls._meta.database.create_table(cls)
     
     @classmethod
     def drop_table(cls):
-        cls.database.drop_table(cls)
+        cls._meta.database.drop_table(cls)
     
     @classmethod
     def select(cls, query=None):
@@ -930,6 +954,20 @@ class Model(object):
     @classmethod
     def delete(cls, **query):
         return DeleteQuery(cls, **query)
+
+    @classmethod
+    def create(cls, **query):
+        inst = cls(**query)
+        inst.save()
+        return inst
+
+    @classmethod
+    def get_or_create(cls, **query):
+        try:
+            inst = cls.get(**query)
+        except StopIteration:
+            inst = cls.create(**query)
+        return inst
     
     @classmethod            
     def get(cls, *args, **kwargs):
